@@ -7,11 +7,15 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 )
 
 const testTimeout = 30 * time.Minute
 
+type buildSetupFn func(build *common.Build)
+
 func RunBuildReturningOutput(t *testing.T, build *common.Build) (string, error) {
 	buf := new(bytes.Buffer)
 	err := RunBuildWithTrace(t, build, &common.Trace{Writer: buf})
@@ -41,9 +45,8 @@ func RunBuild(t *testing.T, build *common.Build) error {
 	return err
 }
 
-// OnUserStage executes the provided function when the CurrentStage() enters
-// a non-predefined stage.
-func OnUserStage(build *common.Build, fn func()) func() {
+// OnStage executes the provided function when the provided stage is entered.
+func OnStage(build *common.Build, stage string, fn func()) func() {
 	exit := make(chan struct{})
 
 	go func() {
@@ -52,8 +55,9 @@ func OnUserStage(build *common.Build, fn func()) func() {
 			case <-exit:
 				return
 
-			case <-time.After(time.Second):
-				if strings.HasPrefix(string(build.CurrentStage()), "step_") {
+			case <-time.After(200 * time.Millisecond):
+				currentStage := string(build.CurrentStage())
+				if strings.HasPrefix(currentStage, stage) {
 					fn()
 					return
 				}
@@ -65,3 +69,22 @@ func OnUserStage(build *common.Build, fn func()) func() {
 		close(exit)
 	}
 }
+
+// OnUserStage executes the provided function when the CurrentStage() enters
+// a non-predefined stage.
+func OnUserStage(build *common.Build, fn func()) func() {
+	return OnStage(build, "step_", fn)
+}
+
+type baseJobGetter func() (common.JobResponse, error)
+
+// getJobResponseWithCommands is a wrapper that will decorate a JobResponse getter
+// like common.GetRemoteSuccessfulBuild with a custom commands list
+func getJobResponseWithCommands(t *testing.T, baseJobGetter baseJobGetter, commands ...string) common.JobResponse {
+	jobResponse, err := baseJobGetter()
+	require.NoError(t, err)
+
+	jobResponse.Steps[0].Script = commands
+
+	return jobResponse
+}
