@@ -17,6 +17,7 @@ type machineProvider struct {
 	name        string
 	machine     docker.Machine
 	details     machinesDetails
+	runners     runnersDetails
 	lock        sync.RWMutex
 	acquireLock sync.Mutex
 	// provider stores a real executor that is used to start run the builds
@@ -31,43 +32,73 @@ type machineProvider struct {
 }
 
 func (m *machineProvider) machineDetails(name string, acquire bool) *machineDetails {
+	details := m.ensureDetails(name)
+	if acquire {
+		details = m.tryAcquireMachineDetails(details)
+	}
+
+	return details
+}
+
+func (m *machineProvider) ensureDetails(name string) *machineDetails {
 	m.lock.Lock()
 	defer m.lock.Unlock()
 
 	details, ok := m.details[name]
 	if !ok {
+		now := time.Now()
 		details = &machineDetails{
 			Name:      name,
-			Created:   time.Now(),
-			Used:      time.Now(),
-			LastSeen:  time.Now(),
+			Created:   now,
+			Used:      now,
+			LastSeen:  now,
 			UsedCount: 1, // any machine that we find we mark as already used
 			State:     machineStateIdle,
 		}
 		m.details[name] = details
 	}
 
-	if acquire {
-		if details.isUsed() {
-			return nil
-		}
-		details.State = machineStateAcquired
-	}
-
 	return details
 }
 
-func (m *machineProvider) create(config *common.RunnerConfig, state machineState) (details *machineDetails, errCh chan error) {
+var errNoConfig = errors.New("no runner config specified")
+
+func (m *machineProvider) runnerMachinesCoordinator(config *common.RunnerConfig) (*runnerMachinesCoordinator, error) {
+	if config == nil {
+		return nil, errNoConfig
+	}
+
+	m.lock.Lock()
+	defer m.lock.Unlock()
+
+	details, ok := m.runners[config.GetToken()]
+	if !ok {
+		details = newRunnerMachinesCoordinator()
+		m.runners[config.GetToken()] = details
+	}
+
+	return details, nil
+}
+
+func (m *machineProvider) create(config *common.RunnerConfig, state machineState) (*machineDetails, chan error) {
 	name := newMachineName(config)
-	details = m.machineDetails(name, true)
+	details := m.machineDetails(name, true)
+	m.lock.Lock()
 	details.State = machineStateCreating
 	details.UsedCount = 0
 	details.RetryCount = 0
 	details.LastSeen = time.Now()
-	errCh = make(chan error, 1)
+	m.lock.Unlock()
+	errCh := make(chan error, 1)
 
-	// Create machine asynchronously
-	go func() {
+	// Create machine with the required configuration asynchronously
+	coordinator, err := m.runnerMachinesCoordinator(config)
+	if err != nil {
+		errCh <- err
+		return nil, errCh
+	}
+
+	go coordinator.waitForGrowthCapacity(config.Machine.MaxGrowthRate, func() {
 		started := time.Now()
 		err := m.machine.Create(config.Machine.MachineDriver, details.Name, config.Machine.MachineOptions...)
 		for i := 0; i < 3 && err != nil; i++ {
@@ -84,22 +115,31 @@ func (m *machineProvider) create(config *common.RunnerConfig, state machineState
 				WithField("time", time.Since(started)).
 				WithError(err).
 				Errorln("Machine creation failed")
-			m.remove(details.Name, "Failed to create")
+			_ = m.remove(details.Name, "Failed to create")
 		} else {
+			m.lock.Lock()
 			details.State = state
 			details.Used = time.Now()
+			m.lock.Unlock()
 			creationTime := time.Since(started)
+			m.lock.RLock()
 			logrus.WithField("duration", creationTime).
 				WithField("name", details.Name).
 				WithField("now", time.Now()).
 				WithField("retries", details.RetryCount).
 				Infoln("Machine created")
+			m.lock.RUnlock()
 			m.totalActions.WithLabelValues("created").Inc()
 			m.creationHistogram.Observe(creationTime.Seconds())
+
+			// Signal that a new machine is available. When there's contention, there's no guarantee between the
+			// ordering of reading from errCh and the availability check.
+			coordinator.addAvailableMachine()
 		}
 		errCh <- err
-	}()
-	return
+	})
+
+	return details, errCh
 }
 
 func (m *machineProvider) findFreeMachine(skipCache bool, machines ...string) (details *machineDetails) {
@@ -114,7 +154,7 @@ func (m *machineProvider) findFreeMachine(skipCache bool, machines ...string) (d
 		// Check if node is running
 		canConnect := m.machine.CanConnect(name, skipCache)
 		if !canConnect {
-			m.remove(name, "machine is unavailable")
+			_ = m.remove(name, "machine is unavailable")
 			continue
 		}
 		return details
@@ -123,18 +163,80 @@ func (m *machineProvider) findFreeMachine(skipCache bool, machines ...string) (d
 	return nil
 }
 
-func (m *machineProvider) useMachine(config *common.RunnerConfig) (details *machineDetails, err error) {
+func (m *machineProvider) findFreeExistingMachine(config *common.RunnerConfig) (*machineDetails, error) {
 	machines, err := m.loadMachines(config)
 	if err != nil {
-		return
+		return nil, err
 	}
-	details = m.findFreeMachine(true, machines...)
-	if details == nil {
-		var errCh chan error
-		details, errCh = m.create(config, machineStateAcquired)
-		err = <-errCh
+
+	return m.findFreeMachine(true, machines...), nil
+}
+
+func (m *machineProvider) useMachine(config *common.RunnerConfig) (*machineDetails, error) {
+	details, err := m.findFreeExistingMachine(config)
+	if err != nil || details != nil {
+		return details, err
 	}
-	return
+
+	return m.createAndAcquireMachine(config)
+}
+
+func (m *machineProvider) createAndAcquireMachine(config *common.RunnerConfig) (*machineDetails, error) {
+	coordinator, err := m.runnerMachinesCoordinator(config)
+	if err != nil {
+		return nil, err
+	}
+
+	newDetails, errCh := m.create(config, machineStateIdle)
+	// Use either a free machine, or the created machine; whichever comes first. There's no guarantee that the created
+	// machine can be used by us because between the time the machine is created, and the acquisition of the machine,
+	// another goroutine may have found it via findFreeMachine and acquired it.
+	var details *machineDetails
+	for details == nil && err == nil {
+		select {
+		case err = <-errCh:
+			if err != nil {
+				return nil, err
+			}
+
+			details = m.tryAcquireMachineDetails(newDetails)
+		case <-coordinator.availableMachineSignal():
+			// Even though the signal is fired and we are *almost* sure that
+			// there's a machine available, let's use the getAvailableMachine
+			// method so that the internal counter is synchonized with what
+			// we are actually doing and so that we can be sure that no other
+			// goroutine that didn't accept the signal and instead used the ticker
+			// hasn't already snatched a machine
+			details, err = m.tryGetFreeExistingMachineFromCoordinator(config, coordinator)
+		case <-time.After(time.Second):
+			details, err = m.tryGetFreeExistingMachineFromCoordinator(config, coordinator)
+		}
+	}
+
+	return details, err
+}
+
+func (m *machineProvider) tryGetFreeExistingMachineFromCoordinator(
+	config *common.RunnerConfig,
+	coordinator *runnerMachinesCoordinator,
+) (*machineDetails, error) {
+	if coordinator.getAvailableMachine() {
+		return m.findFreeExistingMachine(config)
+	}
+
+	return nil, nil
+}
+
+func (m *machineProvider) tryAcquireMachineDetails(details *machineDetails) *machineDetails {
+	m.lock.Lock()
+	defer m.lock.Unlock()
+
+	if details.isUsed() {
+		return nil
+	}
+
+	details.State = machineStateAcquired
+	return details
 }
 
 func (m *machineProvider) retryUseMachine(config *common.RunnerConfig) (details *machineDetails, err error) {
@@ -227,7 +329,11 @@ func (m *machineProvider) remove(machineName string, reason ...interface{}) erro
 	return nil
 }
 
-func (m *machineProvider) updateMachine(config *common.RunnerConfig, data *machinesData, details *machineDetails) error {
+func (m *machineProvider) updateMachine(
+	config *common.RunnerConfig,
+	data *machinesData,
+	details *machineDetails,
+) error {
 	if details.State != machineStateIdle {
 		return nil
 	}
@@ -251,7 +357,10 @@ func (m *machineProvider) updateMachine(config *common.RunnerConfig, data *machi
 	return nil
 }
 
-func (m *machineProvider) updateMachines(machines []string, config *common.RunnerConfig) (data machinesData, validMachines []string) {
+func (m *machineProvider) updateMachines(
+	machines []string,
+	config *common.RunnerConfig,
+) (data machinesData, validMachines []string) {
 	data.Runner = config.ShortDescription()
 	validMachines = make([]string, 0, len(machines))
 
@@ -263,7 +372,7 @@ func (m *machineProvider) updateMachines(machines []string, config *common.Runne
 		if err == nil {
 			validMachines = append(validMachines, name)
 		} else {
-			m.remove(details.Name, err)
+			_ = m.remove(details.Name, err)
 		}
 
 		data.Add(details)
@@ -282,6 +391,10 @@ func (m *machineProvider) createMachines(config *common.RunnerConfig, data *mach
 			// Limit maximum number of machines
 			break
 		}
+		if data.Creating >= config.Machine.MaxGrowthRate && config.Machine.MaxGrowthRate > 0 {
+			// Prevent excessive growth in the number of machines
+			break
+		}
 		m.create(config, machineStateIdle)
 		data.Creating++
 	}
@@ -333,10 +446,9 @@ func (m *machineProvider) loadMachines(config *common.RunnerConfig) (machines []
 	return
 }
 
-func (m *machineProvider) Acquire(config *common.RunnerConfig) (data common.ExecutorData, err error) {
+func (m *machineProvider) Acquire(config *common.RunnerConfig) (common.ExecutorData, error) {
 	if config.Machine == nil || config.Machine.MachineName == "" {
-		err = fmt.Errorf("missing Machine options")
-		return
+		return nil, fmt.Errorf("missing Machine options")
 	}
 
 	// Lock updating machines, because two Acquires can be run at the same time
@@ -345,7 +457,7 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (data common.Exec
 
 	machines, err := m.loadMachines(config)
 	if err != nil {
-		return
+		return nil, err
 	}
 
 	// Update a list of currently configured machines
@@ -358,6 +470,7 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (data common.Exec
 		WithField("runner", config.ShortDescription()).
 		WithField("minIdleCount", config.Machine.GetIdleCount()).
 		WithField("maxMachines", config.Limit).
+		WithField("maxMachineCreate", config.Machine.MaxGrowthRate).
 		WithField("time", time.Now()).
 		Debugln("Docker Machine Details")
 	machinesData.writeDebugInformation()
@@ -365,18 +478,21 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (data common.Exec
 	// Try to find a free machine
 	details := m.findFreeMachine(false, validMachines...)
 	if details != nil {
-		data = details
-		return
+		return details, nil
 	}
 
 	// If we have a free machines we can process a build
 	if config.Machine.GetIdleCount() != 0 && machinesData.Idle == 0 {
 		err = errors.New("no free machines that can process builds")
 	}
-	return
+	return nil, err
 }
 
-func (m *machineProvider) Use(config *common.RunnerConfig, data common.ExecutorData) (newConfig common.RunnerConfig, newData common.ExecutorData, err error) {
+//nolint:nakedret
+func (m *machineProvider) Use(
+	config *common.RunnerConfig,
+	data common.ExecutorData,
+) (newConfig common.RunnerConfig, newData common.ExecutorData, err error) {
 	// Find a new machine
 	details, _ := data.(*machineDetails)
 	if details == nil || !details.canBeUsed() || !m.machine.CanConnect(details.Name, true) {
@@ -418,22 +534,46 @@ func (m *machineProvider) Use(config *common.RunnerConfig, data common.ExecutorD
 func (m *machineProvider) Release(config *common.RunnerConfig, data common.ExecutorData) {
 	// Release machine
 	details, ok := data.(*machineDetails)
-	if ok {
-		// Mark last used time when is Used
-		if details.State == machineStateUsed {
-			details.Used = time.Now()
-		}
-
-		// Remove machine if we already used it
-		if config != nil && config.Machine != nil &&
-			config.Machine.MaxBuilds > 0 && details.UsedCount >= config.Machine.MaxBuilds {
-			err := m.remove(details.Name, "Too many builds")
-			if err == nil {
-				return
-			}
-		}
-		details.State = machineStateIdle
+	if !ok {
+		return
 	}
+
+	m.lock.Lock()
+	// Mark last used time when is Used
+	if details.State == machineStateUsed {
+		details.Used = time.Now()
+	}
+	m.lock.Unlock()
+
+	// Remove machine if we already used it
+	if config != nil && config.Machine != nil &&
+		config.Machine.MaxBuilds > 0 && details.UsedCount >= config.Machine.MaxBuilds {
+		err := m.remove(details.Name, "Too many builds")
+		if err == nil {
+			return
+		}
+	}
+	m.lock.Lock()
+	details.State = machineStateIdle
+	m.lock.Unlock()
+
+	// Signal pending builds that a new machine is available.
+	if err := m.signalRelease(config); err != nil {
+		return
+	}
+}
+
+func (m *machineProvider) signalRelease(config *common.RunnerConfig) error {
+	coordinator, err := m.runnerMachinesCoordinator(config)
+	if err != nil && err != errNoConfig {
+		return err
+	}
+
+	if err != errNoConfig && coordinator != nil {
+		coordinator.addAvailableMachine()
+	}
+
+	return nil
 }
 
 func (m *machineProvider) CanCreate() bool {
@@ -463,6 +603,7 @@ func newMachineProvider(name, executor string) *machineProvider {
 	return &machineProvider{
 		name:     name,
 		details:  make(machinesDetails),
+		runners:  make(runnersDetails),
 		machine:  docker.NewMachineCommand(),
 		provider: provider,
 		totalActions: prometheus.NewCounterVec(
