@@ -44,7 +44,14 @@ type fakeRemoteExecutor struct {
 	execErr error
 }
 
-func (f *fakeRemoteExecutor) Execute(method string, url *url.URL, config *restclient.Config, stdin io.Reader, stdout, stderr io.Writer, tty bool) error {
+func (f *fakeRemoteExecutor) Execute(
+	method string,
+	url *url.URL,
+	config *restclient.Config,
+	stdin io.Reader,
+	stdout, stderr io.Writer,
+	tty bool,
+) error {
 	f.method = method
 	f.url = url
 	return f.execErr
@@ -258,8 +265,10 @@ func TestAttach(t *testing.T) {
 		b, _ := ioutil.ReadAll(stdin)
 		return string(b) == "sleep 1\n"
 	})
-	mockExecutor.On("Execute", http.MethodPost,
-		urlMatcher, clientConfig, stdinMatcher, nil, nil, false).Return(nil).Once()
+	mockExecutor.
+		On("Execute", http.MethodPost, urlMatcher, clientConfig, stdinMatcher, nil, nil, false).
+		Return(nil).
+		Once()
 
 	opts := &AttachOptions{
 		Namespace:     "test-resource",
@@ -294,7 +303,7 @@ func TestAttachErrorGettingPod(t *testing.T) {
 		Config:        clientConfig,
 	}
 
-	assert.True(t, errors.Is(opts.Run(), err))
+	assert.ErrorIs(t, opts.Run(), err)
 }
 
 func TestAttachPodNotRunning(t *testing.T) {
