@@ -10,7 +10,7 @@ import (
 	"sync"
 	"time"
 
-	"github.com/docker/docker/api/types"
+	"github.com/docker/cli/cli/config/types"
 	"github.com/jpillora/backoff"
 	"golang.org/x/net/context"
 	api "k8s.io/api/core/v1"
@@ -186,6 +186,7 @@ func (s *executor) prepareHelperImage() (helperimage.Info, error) {
 		OSType:         helperimage.OSTypeLinux,
 		Architecture:   "amd64",
 		GitLabRegistry: s.Build.IsFeatureFlagOn(featureflags.GitLabRegistryHelperImage),
+		Shell:          s.Config.Shell,
 	})
 }
 
@@ -292,6 +293,10 @@ func (s *executor) runWithAttach(cmd common.ExecutorCommand) error {
 
 		return err
 	case err := <-podStatusCh:
+		if isKubernetesPodNotFoundError(err) {
+			return err
+		}
+
 		return &common.BuildError{Inner: err}
 	case <-ctx.Done():
 		return fmt.Errorf("build aborted")
@@ -340,7 +345,7 @@ func (s *executor) buildLogPermissionsInitContainer() api.Container {
 
 	return api.Container{
 		Name:            "change-logs-permissions",
-		Image:           s.helperImageInfo.String(),
+		Image:           s.getHelperImage(),
 		Command:         []string{"sh", "-c", chmod},
 		VolumeMounts:    s.getVolumeMounts(),
 		ImagePullPolicy: api.PullPolicy(s.pullPolicy),
@@ -420,6 +425,16 @@ func (s *executor) generateScripts(shell common.Shell) (map[string]string, error
 	return scripts, nil
 }
 
+func (s *executor) Finish(err error) {
+	if isKubernetesPodNotFoundError(err) {
+		// Avoid an additional error message when trying to
+		// cleanup a pod that we know no longer exists
+		s.pod = nil
+	}
+
+	s.AbstractExecutor.Finish(err)
+}
+
 func (s *executor) Cleanup() {
 	s.cleanupResources()
 	s.cleanupServices()
@@ -1095,8 +1110,7 @@ func (s *executor) watchPodStatus(ctx context.Context) <-chan error {
 
 func (s *executor) checkPodStatus() error {
 	pod, err := s.kubeClient.CoreV1().Pods(s.pod.Namespace).Get(s.pod.Name, metav1.GetOptions{})
-	var statusErr *kubeerrors.StatusError
-	if errors.As(err, &statusErr) && statusErr.ErrStatus.Code == http.StatusNotFound {
+	if isKubernetesPodNotFoundError(err) {
 		return err
 	}
 
@@ -1248,6 +1262,14 @@ func (s *executor) checkDefaults() error {
 	return nil
 }
 
+func isKubernetesPodNotFoundError(err error) bool {
+	var statusErr *kubeerrors.StatusError
+	return errors.As(err, &statusErr) &&
+		statusErr.ErrStatus.Code == http.StatusNotFound &&
+		statusErr.ErrStatus.Details != nil &&
+		statusErr.ErrStatus.Details.Kind == "pods"
+}
+
 func newExecutor() *executor {
 	e := &executor{
 		AbstractExecutor: executors.AbstractExecutor{
