@@ -16,6 +16,7 @@ const (
 	UseFastzip                           string = "FF_USE_FASTZIP"
 	GitLabRegistryHelperImage            string = "FF_GITLAB_REGISTRY_HELPER_IMAGE"
 	DisableUmaskForDockerExecutor        string = "FF_DISABLE_UMASK_FOR_DOCKER_EXECUTOR"
+	EnableBashExitCodeCheck              string = "FF_ENABLE_BASH_EXIT_CODE_CHECK"
 )
 
 type FeatureFlag struct {
@@ -64,7 +65,8 @@ var flags = []FeatureFlag{
 		ToBeRemovedWith: "",
 		Description: "When set to `true` Runner tries to direct-download all artifacts instead of proxying " +
 			"through GitLab on a first try. Enabling might result in a download failures due to problem validating " +
-			"TLS certificate of Object Storage if it is enabled by GitLab",
+			"TLS certificate of Object Storage if it is enabled by GitLab. " +
+			"See [Self-signed certificates or custom Certification Authorities](tls-self-signed.md)",
 	},
 	{
 		Name:            SkipNoOpBuildStages,
@@ -125,6 +127,14 @@ var flags = []FeatureFlag{
 			"downloading artifacts). POSIX utility `id` must be installed and operational in the build image " +
 			"for this feature flag. Runner will execute `id` with options `-u` and `-g` to retrieve the UID and GID.",
 	},
+	{
+		Name:            EnableBashExitCodeCheck,
+		DefaultValue:    "false",
+		Deprecated:      false,
+		ToBeRemovedWith: "",
+		Description: "If enabled, bash scripts don't rely solely on `set -e`, but check for a non-zero exit code " +
+			"after each script command is executed.",
+	},
 }
 
 func GetAll() []FeatureFlag {
