@@ -97,6 +97,11 @@ QcB3QuNtGiQZqEuc/+KM21MSvC8cC/bIOaKZlWbKtEV+tsbuIIhng0opJrEw+5Zq
 VqrwIVjbsGaw/NPROth/XDJp5jzpwxnf5HDQhLV04sfdN9IRw005WC+l0f19iG9V
 6qslKJvNR8A8A+RqvyfIJ0gjNzVLQHrZyTsEbC62w1IcxkBG7lR6W7ZCXal1RSKf
 +3OIln1a6DKx+zEzL20uwW5L/5l3FsLwwvOLybX4mAhiyxahADEA
+-----END PKCS7-----`
+
+	// PKCS7 with no certificates
+	testEmptyCertPKCS7 = `-----BEGIN PKCS7-----
+MCcGCSqGSIb3DQEHAqAaMBgCAQExADALBgkqhkiG9w0BBwGgAKEAMQA=
 -----END PKCS7-----`
 
 	testCertPubKey = `-----BEGIN PUBLIC KEY-----
@@ -150,7 +155,8 @@ func TestDefaultBuilder_BuildChainFromTLSConnectionState(t *testing.T) {
 
 				return mock, cleanup
 			},
-			expectedError:       "error while fetching certificates into the CA Chain: couldn't resolve certificates chain from the leaf certificate: test-error",
+			expectedError: "error while fetching certificates into the CA Chain: couldn't resolve certificates " +
+				"chain from the leaf certificate: test-error",
 			expectedChainLength: 0,
 		},
 		"certificates chain prepared properly": {
@@ -179,10 +185,10 @@ func TestDefaultBuilder_BuildChainFromTLSConnectionState(t *testing.T) {
 			builder := NewBuilder(logrus.StandardLogger()).(*defaultBuilder)
 
 			if tc.setupResolverMock != nil {
-				resolver, cleanup := tc.setupResolverMock(t)
+				resolverMock, cleanup := tc.setupResolverMock(t)
 				defer cleanup()
 
-				builder.resolver = resolver
+				builder.resolver = resolverMock
 			}
 
 			TLS := new(tls.ConnectionState)
