@@ -20,15 +20,14 @@ const maskedText = "[MASKED]"
 const defaultBytesLimit = 4 * 1024 * 1024 // 4MB
 
 type Buffer struct {
-	writer        io.WriteCloser
 	lock          sync.RWMutex
-	logFile       *os.File
 	logSize       int
 	logWriter     *bufio.Writer
 	advanceBuffer bytes.Buffer
-	bytesLimit    int
-	finish        chan struct{}
-	checksum      hash.Hash32
+
+	logFile    *os.File
+	bytesLimit int
+	checksum   hash.Hash32
 
 	maskTree *trie.Trie
 }
@@ -53,10 +52,13 @@ func (b *Buffer) SetLimit(size int) {
 }
 
 func (b *Buffer) Size() int {
+	b.lock.RLock()
+	defer b.lock.RUnlock()
+
 	return b.logSize
 }
 
-func (b *Buffer) Reader(offset, n int) (io.ReadSeeker, error) {
+func (b *Buffer) Bytes(offset, n int) ([]byte, error) {
 	b.lock.Lock()
 	defer b.lock.Unlock()
 
@@ -65,26 +67,26 @@ func (b *Buffer) Reader(offset, n int) (io.ReadSeeker, error) {
 		return nil, err
 	}
 
-	return io.NewSectionReader(b.logFile, int64(offset), int64(n)), nil
-}
-
-func (b *Buffer) Bytes(offset, n int) ([]byte, error) {
-	reader, err := b.Reader(offset, n)
-	if err != nil {
-		return nil, err
-	}
-
-	return ioutil.ReadAll(reader)
+	return ioutil.ReadAll(io.NewSectionReader(b.logFile, int64(offset), int64(n)))
 }
 
 func (b *Buffer) Write(data []byte) (n int, err error) {
-	return b.writer.Write(data)
+	b.lock.Lock()
+	defer b.lock.Unlock()
+
+	for _, r := range bytes.Runes(data) {
+		_ = b.writeRuneUnsafe(r)
+	}
+	_ = b.advanceAllUnsafe()
+
+	return len(data), nil
 }
 
 func (b *Buffer) Finish() {
-	// wait for trace to finish
-	_ = b.writer.Close()
-	<-b.finish
+	b.lock.Lock()
+	defer b.lock.Unlock()
+
+	_ = b.advanceAllUnsafe()
 }
 
 func (b *Buffer) Close() {
@@ -102,13 +104,6 @@ func (b *Buffer) advanceAllUnsafe() error {
 	return err
 }
 
-func (b *Buffer) advanceAll() {
-	b.lock.Lock()
-	defer b.lock.Unlock()
-
-	_ = b.advanceAllUnsafe()
-}
-
 // advanceLogUnsafe is assumed to be run every character
 func (b *Buffer) advanceLogUnsafe() error {
 	// advance all if no masking is enabled
@@ -143,10 +138,7 @@ func (b *Buffer) limitExceededMessage() string {
 	)
 }
 
-func (b *Buffer) writeRune(r rune) error {
-	b.lock.Lock()
-	defer b.lock.Unlock()
-
+func (b *Buffer) writeRuneUnsafe(r rune) error {
 	// over trace limit
 	if b.logSize > b.bytesLimit {
 		return io.EOF
@@ -161,7 +153,7 @@ func (b *Buffer) writeRune(r rune) error {
 	}
 
 	// under trace limit
-	if b.logSize <= b.bytesLimit {
+	if b.logSize < b.bytesLimit {
 		return nil
 	}
 
@@ -170,27 +162,6 @@ func (b *Buffer) writeRune(r rune) error {
 	return b.advanceAllUnsafe()
 }
 
-func (b *Buffer) process(pipe *io.PipeReader) {
-	defer func() { _ = pipe.Close() }()
-
-	reader := bufio.NewReader(pipe)
-
-	for {
-		r, s, err := reader.ReadRune()
-		if s <= 0 {
-			break
-		}
-
-		if err == nil {
-			// only write valid characters
-			_ = b.writeRune(r)
-		}
-	}
-
-	b.advanceAll()
-	close(b.finish)
-}
-
 func New() (*Buffer, error) {
 	logFile, err := ioutil.TempFile("", "trace")
 	if err != nil {
@@ -199,15 +170,12 @@ func New() (*Buffer, error) {
 
 	checksum := crc32.NewIEEE()
 
-	reader, writer := io.Pipe()
 	buffer := &Buffer{
-		writer:     writer,
 		bytesLimit: defaultBytesLimit,
-		finish:     make(chan struct{}),
 		logFile:    logFile,
 		checksum:   checksum,
 		logWriter:  bufio.NewWriter(io.MultiWriter(logFile, checksum)),
 	}
-	go buffer.process(reader)
+
 	return buffer, nil
 }
